/*
    main.c: An example of using the BMF_font library
    Copyright (C) 2003  Trenkwalder Markus

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Trenkwalder Markus
    trenki2@gmx.net
*/

#ifdef _WIN32
# define WIN32_LEAN_AND_MEAN
# include <windows.h>
#endif

#include "SDL.h"
#include <GL/gl.h>

#include "BMF_font.h"


/******************************************************************************/

void Enter2D(void) {
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	glOrtho(0, 640, 0, 480, -1, 1);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
}

void Exit2D(void) {
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}

/******************************************************************************/

int main(int argc, char* argv[]) {
	
	BMF_Font *fnt1, *fnt2;
	
	/* Setting up SDL video */
	SDL_Init(SDL_INIT_VIDEO);
	SDL_SetVideoMode(800, 600, 32, SDL_OPENGL);
	
	/* Loading 2 different fonts (no error handling) */
	fnt1 = BMF_Load("data/fnt1.bmf");
	fnt2 = BMF_Load("data/fnt2.bmf");

	glClear(GL_COLOR_BUFFER_BIT);
	Enter2D();
	
	/* Start a text output block with font 1. Current font attributes are used.
	 * Any changes made to the output position, color etc. will be restored
	 * after BMF_End() is called. So BMF_Begin() operates like the GL function
	 * glPushAttrib().
	 */
	BMF_Begin(fnt1);
		BMF_SetColor(1, 0, 0, 1);	/* red color */
		
		/* The text appears in the bottom left corner since the output position 
		 * has not been modified inside or outside the BMF_Begin() block.
		 */
		BMF_Print("Hello! This text appears in bottom left corner. ");
		BMF_Print("This text will be print after the above.");
		
		/* Set a new text output position. Note: Enter2D switched OpenGL to 2D
		 * mode with virtual window size of 640x480 pixels. This is absolutely
		 * indipendent from the effective screen resolution.
		 * The coordinates given to BMF_SetPos() are coordinates in the
		 * virtual windows coordinate system (origin is the left bottom corner).
		 */
		BMF_SetPos(100, 320);	
	
		BMF_SetColor(0, 1, 0, 1);	/* now use a green color */
		BMF_Print("BMF_Print works just like printf(): %s %.4f", "Can you see?", 10.0 / 3.0);
	BMF_End();
	
	/* Now use the second font that we loaded */
	BMF_Begin(fnt2);
		/* Any changes made in the previous text output block do not affect
		 * this one so the output color will be the default one (white).
		 * The same goes for the output position and all other states that
		 * affect the output.
		 */
		BMF_SetRotation(30);
		BMF_SetColor(1, 1, 1, 0.5);	/* transparent text is also possible*/
		BMF_Print("Hey! This is a transparent text with another font.");
		
		/* Center of the virtual screen which is set to 640x480 */
		BMF_SetPos(320, 240);
		
		/* Adjust the alignment settings so the text will be centered */
		BMF_SetVAlign(BMF_ALIGN_CENTER);
		BMF_SetHAlign(BMF_ALIGN_CENTER);
		
		BMF_SetScale(1, 2);		/* Set a scaling factor */
		BMF_SetRotation(45);	/* Rotate the text a bit */
		
		BMF_SetColor(1, 1, 0, 1);
		BMF_Print("This text is rotated and scaled");
		
		
		/* The position needs to be set again since we want the text to be
		 * output in the center of the screen
		 */
 		BMF_SetPos(320, 240); 
		BMF_SetRotation(0);		/* Reset the rotation */
		BMF_SetScale(2, 2);
		BMF_SetColor(0, 0, 1, 0.75);
		BMF_Print("Centered, blue and transparent text");
		
	BMF_End();
	
	/* Set some attributes. Note: setting attributes outside a text output
	 * block will affect the default state. See below
	 */
	BMF_SetColor(1, 0, 1, 1);
	BMF_SetPos(100, 100);
	
	/* output something with font 1 again */
	BMF_Begin(fnt1);
		BMF_Print("Not white and not located in the bottom left corner");
	BMF_End();

	/* As you can see above it can be tedious if you only have to display some
	 * text. Therefore there exist some shortcut macros.
	 */
	BMF_SetColor(0, 1, 1, 1);
	BMF_SetPos(200, 200);
	
	/* Will use font 1 and the attributes set above */
	BMF_Printf(fnt1, "Text using a shortcut macro!");
	BMF_Printfp(fnt2, 200, 250, "You can also specify a position using a macro");
	
	BMF_SetRotation(-20);
	/* Note: The output position used will be 200, 200 since it is not changed
	 * by the macros as the BMF_Print() function does
	 * (may be changed in future versions)
	 */
	BMF_Printfc(fnt2, 1, 0, 0, 1, "Also colors can be used with macros");
	
	/* If you have to display a lot of text you shouldn't use the macros since 
	 * BMF_Begin() and BMF_End() are called each time. They change the OpenGL
	 * render state and transformation matrices which can be expensive if done
	 * repeatedly. For more performance use only one BMF_Begin/BMF_End pair
	 * per font.
	 */
	 
	 /* The outputed text looks blurred. This is because the virtual coordinate
	  * systems resolution is 640x480 and the windows size is set to 800x600.
	  * Try changing the window size t0 640x480 and you will get sharp edges!
	  */
	
	Exit2D();
	SDL_GL_SwapBuffers();

	{
		SDL_Event event;
		while ( SDL_WaitEvent(&event) && event.type != SDL_QUIT );
	}
	
	/* Cleaning up */
	BMF_Free(fnt1);
	BMF_Free(fnt2);
	
	SDL_Quit();
	return 0;
}

/******************************************************************************/

